#include <opengl-playground/mesh.hpp>
#include <opengl-playground/globalcanvassettings.hpp>

#include <glm/gtc/matrix_transform.hpp>
#include <iostream>

Mesh::Mesh(std::vector<Vertex> vertices, std::vector<unsigned int> indices,
	   std::vector<std::shared_ptr<OpenGlTexture> > textures,
	   std::shared_ptr<OpenGlShaderProgram> shader) :
	OpenGlGraphics(shader)
{
	this->vertices = vertices;
	this->indices = indices;
	this->textures = textures;
}

void Mesh::realize()
{
	glGenVertexArrays(1, &this->vao);
	glGenBuffers(1, &this->vbo);
	glGenBuffers(1, &this->ebo);

	glBindVertexArray(this->vao);

	glBindBuffer(GL_ARRAY_BUFFER, this->vbo);
	glBufferData(GL_ARRAY_BUFFER, this->vertices.size() * sizeof(Vertex), &vertices[0], GL_STATIC_DRAW);

	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, this->ebo);
	glBufferData(GL_ELEMENT_ARRAY_BUFFER, indices.size() * sizeof(unsigned int), &indices[0], GL_STATIC_DRAW);

	glEnableVertexAttribArray(0);
	glVertexAttribPointer(0, 3, GL_FLOAT, GL_FALSE, sizeof(Vertex), (void *)0);
	glEnableVertexAttribArray(1);
	glVertexAttribPointer(1, 3, GL_FLOAT, GL_FALSE, sizeof(Vertex), (void *)offsetof(Vertex, Normal));
	glEnableVertexAttribArray(2);
	glVertexAttribPointer(2, 2, GL_FLOAT, GL_FALSE, sizeof(Vertex), (void *)offsetof(Vertex, TextureCoords));

	glBindVertexArray(0);
	std::cout << "Realizing Mesh" << std::endl;
}

void Mesh::render()
{
	this->render(this->model_matrix, this->normal_matrix);
}

void Mesh::render(const glm::mat4 &model_matrix, const glm::mat4 &normal_matrix)
{
	this->shaderprog->use();
	this->shaderprog->setUniformMat4("model_matrix", model_matrix);
	this->shaderprog->setUniformMat4("normal_matrix", normal_matrix);
	this->shaderprog->setUniformMat4("projection_view_matrix", GlobalCanvasSettings::getPVMatrix());
	this->shaderprog->setUniformVec3("camera_position", GlobalCanvasSettings::getCameraPosition());

	int spec_num = 0;
	int diff_num = 0;
	/* Setup the texture samplers */
	for (unsigned int i = 0; i < this->textures.size(); i++) {
		std::string uniform_name;
		textures[i]->use(i);
		switch (textures[i]->getType()) {
		case TEXTURE_DIFFUSE:
			uniform_name = "material.diffuse" + std::to_string(diff_num++);
			this->shaderprog->setUniformInt(uniform_name, i);
			break;
		case TEXTURE_SPECULAR:
			uniform_name = "material.specular" + std::to_string(spec_num++);
			this->shaderprog->setUniformInt(uniform_name, i);
			break;
		default:
			break;
		}
	}

	glBindVertexArray(this->vao);
	glDrawElements(GL_TRIANGLES, indices.size(), GL_UNSIGNED_INT, 0);
	glBindVertexArray(0);
}




